from curia.session import Session


def create_session(env: str, api_token: str, debug: bool = False) -> Session:
    """Utility function for formatting of new session

    Args:
        env (str): Production, development, or staging environment.
        api_token (str): Generated API token. Run `dbutils.secrets.get(scope='your_id', key='ALEDADE_PROD_API_KEY')` to get the API token.
        debug (bool, optional): Whether to have a debug session Defaults to False.

    Returns:
        Session: The new session.
    """
    insert_stage = f"{env}." if env in ["stage", "dev"] else ""
    host = f"https://api.{insert_stage}curia.ai"
    session = Session(
        api_token=api_token,
        host=host,
        debug=debug,
    )
    return session


from curia.modeling.metrics import MetricsInterface
from curia.modeling.model_interface import ModelInterface
from curia.modeling.trained_model_job import TrainedModelJob
from curia.modeling.predicted_model_job import PredictedModelJob



from typing import (
    Any,
    List,
    Dict,
    Callable,
    Tuple,
)

import mlflow

from curia.experiments.mlflow import MLFlowLogger
from curia.experiments.utils import evaluate_model_on_datasets


def load_mlflow_model(run_id: str) -> Any:
    """
    Load a model from MLFlow using the run_id.

    Parameters:
    run_id (str): The id of the run from which to load the model.

    Returns:
    model: The loaded model.
    """
    try:
        return mlflow.pyfunc.load_model(f"runs:/{run_id}/model")
    except Exception as e:
        raise RuntimeError(f"Could not load model from run {run_id}.") from e


def calculate_metrics(
    run_id: str,
    datasets: Dict[str, Tuple],
    label_col: str,
    prediction_function: Callable,
    metric_func: Callable,
) -> Dict[str, float]:
    """
    Load a model and calculate the validation metrics.

    Parameters:
    run_id (str): The id of the run from which to load the model.
    datasets (Dict[str, Tuple]): The datasets to use for validation.
    label_col (str): The name of the column containing the labels.
    prediction_function (Callable): The function to use to make predictions.
    metric_func (Callable): The function to use to compute the metrics.

    Returns:
    metrics_dict (Dict[str, float]): A dictionary of metric names and their computed values.
    """
    model = load_mlflow_model(run_id)
    try:
        return evaluate_model_on_datasets(
            model=model,
            datasets=datasets,
            label_col=label_col,
            prediction_function=prediction_function,
            metric_func=metric_func,
        )
    except Exception as e:
        raise RuntimeError(
            f"Failed to calculate metrics for model from run {run_id}."
        ) from e


def log_metrics(
    run_id: str, metrics_dict: Dict[str, float], tags: Dict[str, str]
) -> None:
    """
    Log the validation metrics to MLFlow.

    Parameters:
    run_id (str): The id of the run from which to load the model.
    metrics_dict (Dict[str, float]): The metrics to log.
    tags (Dict[str, str]): The tags to associate with the logged metrics in MLFlow.
    """

    @MLFlowLogger(run_name=run_id, tags=tags, is_nested=True)
    def log_results() -> Dict[str, float]:
        """
        A decorated function to log the validation results.

        This function is designed to be decorated by MLFlowLogger, which logs
        the results to MLFlow when the function is called.

        Returns:
        metrics_dict (Dict[str, float]): A dictionary of metric names and their computed values.
        """
        return metrics_dict

    log_results()


def track_multiple_experiments(
    run_ids: List[str],
    datasets: Dict[str, Tuple],
    label_col: str,
    tags: Dict[str, str],
    prediction_function: Callable,
    metric_func: Callable,
) -> None:
    """
    Track multiple experiments by logging their validation metrics to MLFlow.

    Parameters:
    run_ids (List[str]): A list of run ids from which to load the models.
    datasets (Dict[str, Tuple]): The datasets to use for validation.
    tags (Dict[str, str]): The tags to associate with the logged metrics in MLFlow.
    metric_func (Callable): The function to use to compute the metrics.
    """
    for run_id in run_ids:
        metrics_dict = calculate_metrics(
            run_id, datasets, label_col, prediction_function, metric_func
        )
        log_metrics(run_id, metrics_dict, tags)



# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 3.11.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'str',
        'name': 'str',
        'description': 'str',
        'tags': 'list[str]',
        'type': 'object',
        'model_type': 'str',
        'outcome_type': 'object',
        'treatment_type': 'object',
        'location': 'str',
        'snowflake_location': 'str',
        'file_content_type': 'str',
        'file_size': 'str',
        'file_type': 'str',
        'row_count': 'str',
        'column_count': 'float',
        'status': 'str',
        'is_downloadable': 'bool',
        'organization_id': 'str',
        'last_updated_by': 'str',
        'created_by': 'str',
        'created_at': 'datetime',
        'updated_at': 'datetime',
        'archived_at': 'datetime',
        'version': 'float'
    }

    attribute_map = {
        'id': 'id',
        'name': 'name',
        'description': 'description',
        'tags': 'tags',
        'type': 'type',
        'model_type': 'modelType',
        'outcome_type': 'outcomeType',
        'treatment_type': 'treatmentType',
        'location': 'location',
        'snowflake_location': 'snowflakeLocation',
        'file_content_type': 'fileContentType',
        'file_size': 'fileSize',
        'file_type': 'fileType',
        'row_count': 'rowCount',
        'column_count': 'columnCount',
        'status': 'status',
        'is_downloadable': 'isDownloadable',
        'organization_id': 'organizationId',
        'last_updated_by': 'lastUpdatedBy',
        'created_by': 'createdBy',
        'created_at': 'createdAt',
        'updated_at': 'updatedAt',
        'archived_at': 'archivedAt',
        'version': 'version'
    }

    def __init__(self, id=None, name=None, description=None, tags=None, type=None, model_type=None, outcome_type=None, treatment_type=None, location=None, snowflake_location=None, file_content_type=None, file_size=None, file_type=None, row_count=None, column_count=None, status=None, is_downloadable=None, organization_id=None, last_updated_by=None, created_by=None, created_at=None, updated_at=None, archived_at=None, version=None):  # noqa: E501
        """ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto - a model defined in Swagger"""  # noqa: E501
        self._id = None
        self._name = None
        self._description = None
        self._tags = None
        self._type = None
        self._model_type = None
        self._outcome_type = None
        self._treatment_type = None
        self._location = None
        self._snowflake_location = None
        self._file_content_type = None
        self._file_size = None
        self._file_type = None
        self._row_count = None
        self._column_count = None
        self._status = None
        self._is_downloadable = None
        self._organization_id = None
        self._last_updated_by = None
        self._created_by = None
        self._created_at = None
        self._updated_at = None
        self._archived_at = None
        self._version = None
        self.discriminator = None
        if id is not None:
            self.id = id
        self.name = name
        if description is not None:
            self.description = description
        if tags is not None:
            self.tags = tags
        self.type = type
        if model_type is not None:
            self.model_type = model_type
        if outcome_type is not None:
            self.outcome_type = outcome_type
        if treatment_type is not None:
            self.treatment_type = treatment_type
        if location is not None:
            self.location = location
        if snowflake_location is not None:
            self.snowflake_location = snowflake_location
        if file_content_type is not None:
            self.file_content_type = file_content_type
        if file_size is not None:
            self.file_size = file_size
        if file_type is not None:
            self.file_type = file_type
        if row_count is not None:
            self.row_count = row_count
        if column_count is not None:
            self.column_count = column_count
        if status is not None:
            self.status = status
        if is_downloadable is not None:
            self.is_downloadable = is_downloadable
        self.organization_id = organization_id
        if last_updated_by is not None:
            self.last_updated_by = last_updated_by
        if created_by is not None:
            self.created_by = created_by
        if created_at is not None:
            self.created_at = created_at
        if updated_at is not None:
            self.updated_at = updated_at
        if archived_at is not None:
            self.archived_at = archived_at
        if version is not None:
            self.version = version

    @property
    def id(self):
        """Gets the id of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501


        :return: The id of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.


        :param id: The id of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def name(self):
        """Gets the name of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501


        :return: The name of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.


        :param name: The name of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :type: str
        """
        if name is None:
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def description(self):
        """Gets the description of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501


        :return: The description of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.


        :param description: The description of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def tags(self):
        """Gets the tags of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501


        :return: The tags of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :rtype: list[str]
        """
        return self._tags

    @tags.setter
    def tags(self, tags):
        """Sets the tags of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.


        :param tags: The tags of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :type: list[str]
        """

        self._tags = tags

    @property
    def type(self):
        """Gets the type of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501


        :return: The type of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :rtype: object
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.


        :param type: The type of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :type: object
        """
        if type is None:
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501

        self._type = type

    @property
    def model_type(self):
        """Gets the model_type of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501


        :return: The model_type of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._model_type

    @model_type.setter
    def model_type(self, model_type):
        """Sets the model_type of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.


        :param model_type: The model_type of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :type: str
        """
        allowed_values = ["risk", "impactability"]  # noqa: E501
        if model_type not in allowed_values:
            raise ValueError(
                "Invalid value for `model_type` ({0}), must be one of {1}"  # noqa: E501
                .format(model_type, allowed_values)
            )

        self._model_type = model_type

    @property
    def outcome_type(self):
        """Gets the outcome_type of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501


        :return: The outcome_type of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :rtype: object
        """
        return self._outcome_type

    @outcome_type.setter
    def outcome_type(self, outcome_type):
        """Sets the outcome_type of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.


        :param outcome_type: The outcome_type of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :type: object
        """

        self._outcome_type = outcome_type

    @property
    def treatment_type(self):
        """Gets the treatment_type of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501


        :return: The treatment_type of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :rtype: object
        """
        return self._treatment_type

    @treatment_type.setter
    def treatment_type(self, treatment_type):
        """Sets the treatment_type of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.


        :param treatment_type: The treatment_type of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :type: object
        """

        self._treatment_type = treatment_type

    @property
    def location(self):
        """Gets the location of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501


        :return: The location of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._location

    @location.setter
    def location(self, location):
        """Sets the location of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.


        :param location: The location of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :type: str
        """

        self._location = location

    @property
    def snowflake_location(self):
        """Gets the snowflake_location of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501


        :return: The snowflake_location of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._snowflake_location

    @snowflake_location.setter
    def snowflake_location(self, snowflake_location):
        """Sets the snowflake_location of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.


        :param snowflake_location: The snowflake_location of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :type: str
        """

        self._snowflake_location = snowflake_location

    @property
    def file_content_type(self):
        """Gets the file_content_type of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501


        :return: The file_content_type of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._file_content_type

    @file_content_type.setter
    def file_content_type(self, file_content_type):
        """Sets the file_content_type of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.


        :param file_content_type: The file_content_type of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :type: str
        """

        self._file_content_type = file_content_type

    @property
    def file_size(self):
        """Gets the file_size of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501


        :return: The file_size of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._file_size

    @file_size.setter
    def file_size(self, file_size):
        """Sets the file_size of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.


        :param file_size: The file_size of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :type: str
        """

        self._file_size = file_size

    @property
    def file_type(self):
        """Gets the file_type of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501


        :return: The file_type of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._file_type

    @file_type.setter
    def file_type(self, file_type):
        """Sets the file_type of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.


        :param file_type: The file_type of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :type: str
        """

        self._file_type = file_type

    @property
    def row_count(self):
        """Gets the row_count of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501


        :return: The row_count of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._row_count

    @row_count.setter
    def row_count(self, row_count):
        """Sets the row_count of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.


        :param row_count: The row_count of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :type: str
        """

        self._row_count = row_count

    @property
    def column_count(self):
        """Gets the column_count of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501


        :return: The column_count of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :rtype: float
        """
        return self._column_count

    @column_count.setter
    def column_count(self, column_count):
        """Sets the column_count of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.


        :param column_count: The column_count of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :type: float
        """

        self._column_count = column_count

    @property
    def status(self):
        """Gets the status of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501


        :return: The status of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.


        :param status: The status of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :type: str
        """

        self._status = status

    @property
    def is_downloadable(self):
        """Gets the is_downloadable of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501


        :return: The is_downloadable of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :rtype: bool
        """
        return self._is_downloadable

    @is_downloadable.setter
    def is_downloadable(self, is_downloadable):
        """Sets the is_downloadable of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.


        :param is_downloadable: The is_downloadable of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :type: bool
        """

        self._is_downloadable = is_downloadable

    @property
    def organization_id(self):
        """Gets the organization_id of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501


        :return: The organization_id of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._organization_id

    @organization_id.setter
    def organization_id(self, organization_id):
        """Sets the organization_id of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.


        :param organization_id: The organization_id of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :type: str
        """
        if organization_id is None:
            raise ValueError("Invalid value for `organization_id`, must not be `None`")  # noqa: E501

        self._organization_id = organization_id

    @property
    def last_updated_by(self):
        """Gets the last_updated_by of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501


        :return: The last_updated_by of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._last_updated_by

    @last_updated_by.setter
    def last_updated_by(self, last_updated_by):
        """Sets the last_updated_by of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.


        :param last_updated_by: The last_updated_by of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :type: str
        """

        self._last_updated_by = last_updated_by

    @property
    def created_by(self):
        """Gets the created_by of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501


        :return: The created_by of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.


        :param created_by: The created_by of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :type: str
        """

        self._created_by = created_by

    @property
    def created_at(self):
        """Gets the created_at of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501


        :return: The created_at of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.


        :param created_at: The created_at of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :type: datetime
        """

        self._created_at = created_at

    @property
    def updated_at(self):
        """Gets the updated_at of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501


        :return: The updated_at of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.


        :param updated_at: The updated_at of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :type: datetime
        """

        self._updated_at = updated_at

    @property
    def archived_at(self):
        """Gets the archived_at of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501


        :return: The archived_at of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :rtype: datetime
        """
        return self._archived_at

    @archived_at.setter
    def archived_at(self, archived_at):
        """Sets the archived_at of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.


        :param archived_at: The archived_at of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :type: datetime
        """

        self._archived_at = archived_at

    @property
    def version(self):
        """Gets the version of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501


        :return: The version of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :rtype: float
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.


        :param version: The version of this ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto.  # noqa: E501
        :type: float
        """

        self._version = version

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ModelJobJoinedModelPopulationJoinedDataQueryJoinedDatasetResponseDto):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import dotenv
import click

from curia.infra.container.cli import container
from curia.infra.databricks.cli import databricks

dotenv.load_dotenv()


@click.group()
def cli():
    pass


cli.add_command(container)
cli.add_command(databricks)

if __name__ == "__main__":
    cli()



import functools
from typing import Callable, Dict, Any, Union
from curia.experiments.utils import MLFLowDatabricksCredentials
import mlflow
import os
import pandas as pd

class MLFlowLogger:
    def __init__(self,
                 run_name: str = None,
                 tags: Dict[str, Any] = None,
                 autolog: bool = False,
                 is_nested: bool = False,
                 mlflow_databricks_credentials: MLFLowDatabricksCredentials = None) -> None:
        """
        Refer to using_mlflow_logging for documentation.
        """
        self.run_name = run_name
        self.tags = tags
        self.autolog = autolog
        self.is_nested = is_nested
        if mlflow_databricks_credentials is not None:
            self._set_env_variables(mlflow_databricks_credentials)

    def _set_env_variables(self, mlflow_databricks_credentials: MLFLowDatabricksCredentials) -> None:
        """Helper function for setting environment variables for logging to Databricks.

        Args:
            mlflow_databricks_credentials (MLFLowDatabricksCredentials): The credentials for logging to Databricks.
        """
        os.environ['MLFLOW_TRACKING_URI'] = "databricks"
        os.environ['DATABRICKS_HOST'] = mlflow_databricks_credentials.host
        os.environ['DATABRICKS_TOKEN'] = mlflow_databricks_credentials.token
        mlflow.set_experiment(mlflow_databricks_credentials.experiment_name)

    @staticmethod
    def log_items(items_to_log: Dict[str, Union[float, pd.DataFrame, int, str]]):
        """Called by the decorator to log items to MLFlow.

        Args:
            items_to_log (Dict[str, Union[float, pd.DataFrame]]): A dictionary of items to log to MLFlow.
        """
        if not isinstance(items_to_log, dict):
            raise TypeError("items_to_log must be a dictionary")
        for metric_name, metric_value in items_to_log.items():
            if isinstance(metric_value, pd.DataFrame):
                if ".json" not in metric_name:
                    metric_name += ".json"
                mlflow.log_table(artifact_file=metric_name, data=metric_value)
            else:
                mlflow.log_metric(metric_name, metric_value)

    def __call__(self, func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            if self.autolog:
                mlflow.autolog(log_datasets=False, log_models=True)
            with mlflow.start_run(
                nested=self.is_nested, run_name=self.run_name, tags=self.tags
            ):
                result = func(*args, **kwargs)

                # If the function returns a dictionary, log it.
                if isinstance(result, dict):
                    self.log_items(result)

            return result

        return wrapper


def using_mlflow_logging(
    run_name: str = None,
    tags: Dict[str, Any] = None,
    autolog: bool = False,
    is_nested: bool = False,
    mlflow_databricks_credentials: MLFLowDatabricksCredentials = None,
) -> Callable:
    """
    A convenience function for using the MLFlowLogger class as a decorator.

    Args:
        run_name (str, optional): The name of the run. Defaults to None.
        tags (Dict[str, Any], optional): A dictionary of tags to add to the run. Defaults to None.
        autolog (bool, optional): Whether to use MLFlow's autologging feature. Defaults to False.
        is_nested (bool, optional): Whether the run is nested within another run. Defaults to False.
        mlflow_databricks_credentials (MLFLowDatabricksCredentials, optional): Credentials for logging to Databricks. Defaults to None.

    Returns:
        Callable: A decorator for logging MLFlow runs.
    """

    logger = MLFlowLogger(run_name, tags, autolog, is_nested, mlflow_databricks_credentials)

    def decorator(func: Callable) -> Callable:
        return logger(func)

    return decorator



# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 3.11.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class PersonSet(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'logic': 'str',
        'expression': 'BooleanExpression',
        'subsets': 'list[PersonSet]',
        'enabled': 'bool'
    }

    attribute_map = {
        'logic': 'logic',
        'expression': 'expression',
        'subsets': 'subsets',
        'enabled': 'enabled'
    }

    def __init__(self, logic=None, expression=None, subsets=None, enabled=None):  # noqa: E501
        """PersonSet - a model defined in Swagger"""  # noqa: E501
        self._logic = None
        self._expression = None
        self._subsets = None
        self._enabled = None
        self.discriminator = None
        if logic is not None:
            self.logic = logic
        if expression is not None:
            self.expression = expression
        if subsets is not None:
            self.subsets = subsets
        if enabled is not None:
            self.enabled = enabled

    @property
    def logic(self):
        """Gets the logic of this PersonSet.  # noqa: E501


        :return: The logic of this PersonSet.  # noqa: E501
        :rtype: str
        """
        return self._logic

    @logic.setter
    def logic(self, logic):
        """Sets the logic of this PersonSet.


        :param logic: The logic of this PersonSet.  # noqa: E501
        :type: str
        """
        allowed_values = ["and", "or", "minus", "left", "right"]  # noqa: E501
        if logic not in allowed_values:
            raise ValueError(
                "Invalid value for `logic` ({0}), must be one of {1}"  # noqa: E501
                .format(logic, allowed_values)
            )

        self._logic = logic

    @property
    def expression(self):
        """Gets the expression of this PersonSet.  # noqa: E501


        :return: The expression of this PersonSet.  # noqa: E501
        :rtype: BooleanExpression
        """
        return self._expression

    @expression.setter
    def expression(self, expression):
        """Sets the expression of this PersonSet.


        :param expression: The expression of this PersonSet.  # noqa: E501
        :type: BooleanExpression
        """

        self._expression = expression

    @property
    def subsets(self):
        """Gets the subsets of this PersonSet.  # noqa: E501


        :return: The subsets of this PersonSet.  # noqa: E501
        :rtype: list[PersonSet]
        """
        return self._subsets

    @subsets.setter
    def subsets(self, subsets):
        """Sets the subsets of this PersonSet.


        :param subsets: The subsets of this PersonSet.  # noqa: E501
        :type: list[PersonSet]
        """

        self._subsets = subsets

    @property
    def enabled(self):
        """Gets the enabled of this PersonSet.  # noqa: E501


        :return: The enabled of this PersonSet.  # noqa: E501
        :rtype: bool
        """
        return self._enabled

    @enabled.setter
    def enabled(self, enabled):
        """Sets the enabled of this PersonSet.


        :param enabled: The enabled of this PersonSet.  # noqa: E501
        :type: bool
        """

        self._enabled = enabled

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PersonSet, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PersonSet):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 3.11.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ModelJobConfig(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'stage': 'str',
        'job_type': 'str',
        'analysis_method': 'str',
        'model_job_id': 'str',
        'client_id': 'str',
        'cohort_window_count': 'float',
        'train_model_job_id': 'str',
        'container_config': 'ContainerConfig',
        'pipeline_config': 'PipelineConfig',
        'require_twelve_month_evidence_period': 'bool'
    }

    attribute_map = {
        'stage': 'stage',
        'job_type': 'job_type',
        'analysis_method': 'analysis_method',
        'model_job_id': 'model_job_id',
        'client_id': 'client_id',
        'cohort_window_count': 'cohort_window_count',
        'train_model_job_id': 'train_model_job_id',
        'container_config': 'container_config',
        'pipeline_config': 'pipeline_config',
        'require_twelve_month_evidence_period': 'require_twelve_month_evidence_period'
    }

    def __init__(self, stage=None, job_type=None, analysis_method=None, model_job_id=None, client_id=None, cohort_window_count=None, train_model_job_id=None, container_config=None, pipeline_config=None, require_twelve_month_evidence_period=None):  # noqa: E501
        """ModelJobConfig - a model defined in Swagger"""  # noqa: E501
        self._stage = None
        self._job_type = None
        self._analysis_method = None
        self._model_job_id = None
        self._client_id = None
        self._cohort_window_count = None
        self._train_model_job_id = None
        self._container_config = None
        self._pipeline_config = None
        self._require_twelve_month_evidence_period = None
        self.discriminator = None
        self.stage = stage
        self.job_type = job_type
        if analysis_method is not None:
            self.analysis_method = analysis_method
        self.model_job_id = model_job_id
        self.client_id = client_id
        if cohort_window_count is not None:
            self.cohort_window_count = cohort_window_count
        if train_model_job_id is not None:
            self.train_model_job_id = train_model_job_id
        self.container_config = container_config
        self.pipeline_config = pipeline_config
        if require_twelve_month_evidence_period is not None:
            self.require_twelve_month_evidence_period = require_twelve_month_evidence_period

    @property
    def stage(self):
        """Gets the stage of this ModelJobConfig.  # noqa: E501


        :return: The stage of this ModelJobConfig.  # noqa: E501
        :rtype: str
        """
        return self._stage

    @stage.setter
    def stage(self, stage):
        """Sets the stage of this ModelJobConfig.


        :param stage: The stage of this ModelJobConfig.  # noqa: E501
        :type: str
        """
        if stage is None:
            raise ValueError("Invalid value for `stage`, must not be `None`")  # noqa: E501

        self._stage = stage

    @property
    def job_type(self):
        """Gets the job_type of this ModelJobConfig.  # noqa: E501


        :return: The job_type of this ModelJobConfig.  # noqa: E501
        :rtype: str
        """
        return self._job_type

    @job_type.setter
    def job_type(self, job_type):
        """Sets the job_type of this ModelJobConfig.


        :param job_type: The job_type of this ModelJobConfig.  # noqa: E501
        :type: str
        """
        if job_type is None:
            raise ValueError("Invalid value for `job_type`, must not be `None`")  # noqa: E501

        self._job_type = job_type

    @property
    def analysis_method(self):
        """Gets the analysis_method of this ModelJobConfig.  # noqa: E501


        :return: The analysis_method of this ModelJobConfig.  # noqa: E501
        :rtype: str
        """
        return self._analysis_method

    @analysis_method.setter
    def analysis_method(self, analysis_method):
        """Sets the analysis_method of this ModelJobConfig.


        :param analysis_method: The analysis_method of this ModelJobConfig.  # noqa: E501
        :type: str
        """

        self._analysis_method = analysis_method

    @property
    def model_job_id(self):
        """Gets the model_job_id of this ModelJobConfig.  # noqa: E501


        :return: The model_job_id of this ModelJobConfig.  # noqa: E501
        :rtype: str
        """
        return self._model_job_id

    @model_job_id.setter
    def model_job_id(self, model_job_id):
        """Sets the model_job_id of this ModelJobConfig.


        :param model_job_id: The model_job_id of this ModelJobConfig.  # noqa: E501
        :type: str
        """
        if model_job_id is None:
            raise ValueError("Invalid value for `model_job_id`, must not be `None`")  # noqa: E501

        self._model_job_id = model_job_id

    @property
    def client_id(self):
        """Gets the client_id of this ModelJobConfig.  # noqa: E501


        :return: The client_id of this ModelJobConfig.  # noqa: E501
        :rtype: str
        """
        return self._client_id

    @client_id.setter
    def client_id(self, client_id):
        """Sets the client_id of this ModelJobConfig.


        :param client_id: The client_id of this ModelJobConfig.  # noqa: E501
        :type: str
        """
        if client_id is None:
            raise ValueError("Invalid value for `client_id`, must not be `None`")  # noqa: E501

        self._client_id = client_id

    @property
    def cohort_window_count(self):
        """Gets the cohort_window_count of this ModelJobConfig.  # noqa: E501


        :return: The cohort_window_count of this ModelJobConfig.  # noqa: E501
        :rtype: float
        """
        return self._cohort_window_count

    @cohort_window_count.setter
    def cohort_window_count(self, cohort_window_count):
        """Sets the cohort_window_count of this ModelJobConfig.


        :param cohort_window_count: The cohort_window_count of this ModelJobConfig.  # noqa: E501
        :type: float
        """

        self._cohort_window_count = cohort_window_count

    @property
    def train_model_job_id(self):
        """Gets the train_model_job_id of this ModelJobConfig.  # noqa: E501


        :return: The train_model_job_id of this ModelJobConfig.  # noqa: E501
        :rtype: str
        """
        return self._train_model_job_id

    @train_model_job_id.setter
    def train_model_job_id(self, train_model_job_id):
        """Sets the train_model_job_id of this ModelJobConfig.


        :param train_model_job_id: The train_model_job_id of this ModelJobConfig.  # noqa: E501
        :type: str
        """

        self._train_model_job_id = train_model_job_id

    @property
    def container_config(self):
        """Gets the container_config of this ModelJobConfig.  # noqa: E501


        :return: The container_config of this ModelJobConfig.  # noqa: E501
        :rtype: ContainerConfig
        """
        return self._container_config

    @container_config.setter
    def container_config(self, container_config):
        """Sets the container_config of this ModelJobConfig.


        :param container_config: The container_config of this ModelJobConfig.  # noqa: E501
        :type: ContainerConfig
        """
        if container_config is None:
            raise ValueError("Invalid value for `container_config`, must not be `None`")  # noqa: E501

        self._container_config = container_config

    @property
    def pipeline_config(self):
        """Gets the pipeline_config of this ModelJobConfig.  # noqa: E501


        :return: The pipeline_config of this ModelJobConfig.  # noqa: E501
        :rtype: PipelineConfig
        """
        return self._pipeline_config

    @pipeline_config.setter
    def pipeline_config(self, pipeline_config):
        """Sets the pipeline_config of this ModelJobConfig.


        :param pipeline_config: The pipeline_config of this ModelJobConfig.  # noqa: E501
        :type: PipelineConfig
        """
        if pipeline_config is None:
            raise ValueError("Invalid value for `pipeline_config`, must not be `None`")  # noqa: E501

        self._pipeline_config = pipeline_config

    @property
    def require_twelve_month_evidence_period(self):
        """Gets the require_twelve_month_evidence_period of this ModelJobConfig.  # noqa: E501


        :return: The require_twelve_month_evidence_period of this ModelJobConfig.  # noqa: E501
        :rtype: bool
        """
        return self._require_twelve_month_evidence_period

    @require_twelve_month_evidence_period.setter
    def require_twelve_month_evidence_period(self, require_twelve_month_evidence_period):
        """Sets the require_twelve_month_evidence_period of this ModelJobConfig.


        :param require_twelve_month_evidence_period: The require_twelve_month_evidence_period of this ModelJobConfig.  # noqa: E501
        :type: bool
        """

        self._require_twelve_month_evidence_period = require_twelve_month_evidence_period

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ModelJobConfig, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ModelJobConfig):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import os
import typing

import click

from curia.api.swagger_client import Workflow, WorkflowTemplate, Task
from curia.infra.task import TaskDefinition
from curia.infra.types import TaskDeployResourceNamingConfig, TaskType
from curia.session import Session


def update_curia_task_and_stub_workflow(task: TaskDefinition,
                                        stage: str,
                                        resource_naming_config: TaskDeployResourceNamingConfig,
                                        task_specific_context: dict) -> None:
    """
    Creates or updates the curia task definition for a task in the curia platform using the curia SDK
    :param task: The task definition
    :param stage: The stage to deploy the task to
    :param resource_naming_config: The resource naming config to use when creating the task
    :param task_specific_context: The task specific context to use when creating the task, identifying external
    resources specific to the task like databricks job IDs or ECS task definitions
    :return:
    """
    session = Session(
        api_token=os.environ["CURIA_API_KEY"],
        host=os.environ["CURIA_API_ENDPOINT"]
    )

    internal_task_name = f"{resource_naming_config.platform_task_prefix}{stage}-{task.task_slug}"

    task_id = _create_or_update_task(
        session=session,
        task=task,
        internal_task_name=internal_task_name,
        task_type=task.task_type,
        task_specific_context=task_specific_context
    )

    workflow_template_id = _create_or_update_workflow_template(
        session,
        stage,
        task,
        task_id,
        resource_naming_config,
        task_specific_context
    )

    workflow_id = _create_or_update_workflow(session, task, internal_task_name, workflow_template_id)

    click.echo(f"""
Created or updated workflow ecosystem {internal_task_name} with data:
workflow_id: {workflow_id}
workflow_template_id: {workflow_template_id}
task_id: {task_id}
    """)


def _create_or_update_workflow(
        session: Session,
        task: TaskDefinition,
        task_name: str,
        workflow_template_id: str) -> str:
    """
    Creates or updates a workflow in the curia platform
    :param session: curia session
    :param task: task definition that is managing the workflow
    :param task_name: name of the workflow
    :param workflow_template_id: id of the workflow template to use with this workflow
    :return:
    """
    # get the workflow, if it exists
    results = session.api_instance.get_many_base_workflow_controller_workflow(
        filter=[f"name||$eq||{task_name}"]
    )
    if len(results.data) == 0:
        # create the workflow
        workflow_id = typing.cast(str, session.api_instance.create_one_base_workflow_controller_workflow(
            Workflow(
                name=task_name,
                description=task.description,
                organization_id=os.environ["CURIA_PRIMARY_ORGANIZATION_ID"],
                template_id=workflow_template_id
            )
        ).id)
    elif len(results.data) == 1:
        existing_workflow = results.data[0]
        workflow_id = typing.cast(str, existing_workflow.id)
        # update the workflow
        existing_workflow.template_id = workflow_template_id
        existing_workflow.name = task_name
        existing_workflow.description = task.description
        session.api_instance.update_one_base_workflow_controller_workflow(
            id=workflow_id,
            body=existing_workflow
        )
    else:
        raise ValueError(f"Found multiple workflows with name {task_name}. This should not "
                         f"happen, please clean up the present workflows in the curia platform")
    return workflow_id


def _create_or_update_workflow_template(
        session: Session,
        env: str,
        task: TaskDefinition,
        task_id: str,
        resource_naming_config: TaskDeployResourceNamingConfig,
        task_specific_context: dict) -> str:
    """
    Creates or updates a workflow template in the curia platform
    :param session: curia session
    :param env: environment to use (dev, stage, prod)
    :param task: task definition that is managing the workflow template
    :param task_id: id of the task to use with this workflow template
    :param resource_naming_config: resource naming config
    :param task_specific_context: task specific context
    :return:
    """
    new_workflow_template = WorkflowTemplate(
        name=f"{resource_naming_config.platform_workflow_prefix}{env}-{task.task_slug}",
        description=task.description,
        parameters={
            **{
                task_input.name: "" for task_input in task.inputs
            }
        },
        definition={
            "taskId": task_id,
            "data": {
                **task.build_workflow_data_block(task.inputs, task_specific_context)
            },
            "opts": {},
            "children": []
        },
        organization_id=os.environ["CURIA_PRIMARY_ORGANIZATION_ID"]
    )
    # get the workflow template, if it exists
    results = session.api_instance.get_many_base_workflow_template_controller_workflow_template(
        filter=[f"name||$eq||{new_workflow_template.name}"]
    ).data
    if len(results) == 0:
        # create the workflow template
        workflow_template_id = typing.cast(str, session.api_instance
                                           .create_one_base_workflow_template_controller_workflow_template(
            new_workflow_template).id)
    elif len(results) == 1:
        workflow_template_id = typing.cast(str, results[0].id)
        # update the workflow template
        session.api_instance.update_one_base_workflow_template_controller_workflow_template(
            id=workflow_template_id,
            body=new_workflow_template
        )
    else:
        raise ValueError(f"Found multiple workflow templates with name {new_workflow_template.name}. This should not "
                         f"happen, please clean up the present workflow templates in the curia platform")
    return workflow_template_id


def _create_or_update_task(
        session: Session,
        task: TaskDefinition,
        internal_task_name: str,
        task_type: TaskType,
        task_specific_context: dict) -> str:
    """
    Creates or updates a task in the curia platform using the curia SDK
    :param session: curia session
    :param task: task definition that is managing the task's deployment
    :param internal_task_name: name of the task
    :param task_type: type of the task
    :param task_specific_context: task specific context
    :return:
    """
    new_task = Task(
        name=internal_task_name,
        description=task.description,
        type=task_type.value,
        inputs=task.build_task_inputs(task.inputs, task_specific_context),
        outputs={
            task_output.name: {
                "descr": task_output.description
            } for task_output in task.outputs
        },
        organization_id=os.environ["CURIA_PRIMARY_ORGANIZATION_ID"],
    )
    # get the task definition, if it exists
    results = session.api_instance.get_many_base_task_controller_task(
        filter=[f"name||$eq||{internal_task_name}"]
    ).data
    if len(results) == 0:
        # create the task definition
        task_id = typing.cast(str, session.api_instance.create_one_base_task_controller_task(new_task).id)
    elif len(results) == 1:
        # update the task definition
        task_id = typing.cast(str, results[0].id)

        session.api_instance.update_one_base_task_controller_task(
            id=task_id,
            body=new_task
        )
    else:
        raise ValueError(
            f"Found multiple tasks with name {internal_task_name}. This should not happen, please clean up the "
            f"present tasks in the curia platform")
    return task_id
